import glob
import os
import json
import pprint
from machines import *

class Config(object):
    def __init__(self):
        self.cfg = {"globals":{},"modes":{}}
        self.cfg["globals"] = {
            "default_title":"BitScope PiLab",
            "colours":{},
            "fonts":{},
            "resolution":[900,506],
            "resolution":[620,480],
            "fullscreen":False,
            "frame_rate":50,
            "widget_sheet":None,
            "widget_rects":{},
            "machines":{},
            "fast_widgets":False
        }

        self.cfg["globals"]["colours"] = {
            "cha" :[255,255,50],
            "chb" :[60,255,60],
            "on"  :[210,215,190],
            "off" :[60,60,60],
            "diff":[90,220,255],
            "red" :[255,0,0],
            "green":[0,255,0],
            "blue":[0,0,255],
            "purple":[255,0,255],
            "pink":[255,180,210],
            "yellow":[255,255,0],
            "aqua":[0,255,255],
            "default":[200,200,210],
            "default_dark":[150,150,155],
            "default_darker":[100,100,105]
        }

        self.cfg["globals"]["fonts"] = {
            "standard":"./font/VeraBd.ttf",
            "mono":"./font/VeraMoBd.ttf"
        }

        self.cfg["globals"]["widget_sheet"] = "Graphics/PNGs/sheet_states_new.png"

        self.cfg["globals"]["widget_rects"] = {
            "tiny"    : [0,0,32,32],
            "sml_h"   : [0,32,48,48],
            "sml"     : [0,80,64,32],
            "std"     : [0,112,96,32],
            "big"     : [0,144,128,64],
            "huge"    : [0,0,256,128],
            "sml_disp": [0,208,128,64],
            "big_disp": [0,272,384,96],
            "huge_disp":[0,0,768,192],
            "led_t"   : [0,464,8,8],
            "led_s"   : [0,0,12,12],
            "led"     : [0,472,16,16],
            "led_data": [0,488,48,24],
            "led_link": [0,512,48,24],
            "led_big" : [0,544,48,48],
            "led_huge": [0,0,64,64]
        }
        self.cfg["globals"]["proto_pos"] = {
            "rounded_button":[0,0],
            "squared_button":[0,30],
            "rounded_panel":[0,42],
            "squared_panel":[0,72],
            "rounded_led":[0,84],
            "squared_led":[0,114]
        } # Squared graphics get to have much smaller slice sizes.
        
        self.cfg["globals"]["default_refs"] = [
            "data",
            "enable",
            "disable",
            "redraw",
            "on",
            "off",
            "colour"
        ]

        self.cfg["globals"]["machines"] = {
            "lobby":Lobby,
            "volt_m":Volt_M,
            "mfg_test":MFG_Test,
            "mfg_post_test":MFG_Post_Test,
            "ttafe":TTAFE,
            "awg_sound":AWG_Sound,
            "scope":Scope
        }

        

        self.cfg["modes"] = {}
        self.load_config_files("./config")
        self.convert_colours(self.cfg)
        self.convert_colour(self.cfg)
        self.convert_rects(self.cfg)
        self.convert_sizes(self.cfg)
        self.convert_fonts(self.cfg)


    def load_config_files(self, directory):
        # Get all the JSON files from the current directory
        cfgs = glob.glob(directory + "/*.json")
        cfgs.sort()
        # Append them all to the config tree
        for f in cfgs:
            f = open(f, 'r')
            j = json.loads(f.read())
            self.update_branch(j['cfg'], self.cfg)
        # Find all folders in the current dir
        for newdir in os.walk(directory):
            if newdir[0] != directory:
                self.load_config_files(newdir[0])
                
    def update_branch(self, fi, cfg):
        for key in fi.keys():
            if key in cfg.keys():
                if isinstance(cfg[key], dict) and isinstance(fi[key], dict): #If both are dicts, step in.
                    self.update_branch(fi[key], cfg[key])
                elif not isinstance(cfg[key], dict) and not isinstance(fi[key], dict):
                    #Otherwise, check that they're both not dicts and update.
                    cfg[key] = fi[key]
            elif isinstance(fi[key], dict):
                cfg[key] = {} #If it's not in the config tree but is a dict in the file; make it a dict...
                self.update_branch(fi[key], cfg[key]) #And step into it!
            else:
                cfg[key] = fi[key] #If it's not in the cfg tree and not meant to be a dict; assign it.

    def convert_colours(self, cfg): #convert tokens
        for k in cfg.keys():
            c1 = k == "cols"
            c2 = "colours" in k
            if c1 or c2 and isinstance(cfg[k], dict):
                for c in cfg[k]:
                    if not isinstance(cfg[k][c], list):
                        cfg[k][c] = self.cfg["globals"]["colours"][cfg[k][c]]
            elif isinstance(cfg[k], dict):
                self.convert_colours(cfg[k])
                
    def convert_colour(self, cfg): # Converts single colours.
        for k in cfg.keys():
            c1 = "colour" in k
            c2 = not ("colours" in k)
            c3 = isinstance(cfg[k], basestring)
            c4 = not ("default_colour" in k)
            if c1 and c2 and c3 and c4:
                cfg[k] = self.cfg['globals']['colours'][cfg[k]]    
            elif isinstance(cfg[k], dict):
                self.convert_colour(cfg[k])
                
    def convert_rects(self, cfg):
        for k in cfg.keys():
            if k == "rect" and not isinstance(cfg[k], list):
                cfg[k] = self.cfg["globals"]["widget_rects"][cfg[k]]
            elif isinstance(cfg[k], dict):
                self.convert_rects(cfg[k])
                
    def convert_sizes(self, cfg):
        for k in cfg.keys():
            if k == "size" and not isinstance(cfg[k], list):
                cfg[k] = self.cfg['globals']['widget_rects'][cfg[k]][2:]
            elif isinstance(cfg[k], dict):
                self.convert_sizes(cfg[k])

    def convert_fonts(self, cfg):
        for k in cfg.keys():
            if k == "font":
                cfg[k] = self.cfg["globals"]["fonts"][cfg[k]]
            elif isinstance(cfg[k], dict):
                self.convert_fonts(cfg[k])
                
    def delete_widgets(self, cfg):
        for k in cfg.keys():
            if k == "delete_widgets":
                for w in cfg[k]:
                    del cfg['layout'][w]
            elif isinstance(cfg[k], dict):
                self.delete_widgets(cfg[k])
                                    
    def include_widgets(self):
        for k in cfg.keys():
            if k == "include_widgets":
                for w in cfg['layout'].keys():
                    if w not in cfg[k]:
                        del cfg['layout'][w]
            elif isinstance(cfg[k], dict):
                self.include_widgets(cfg[k])
