import serial
from serial_tools import Serial_Tools
from tools import *
import time

class Scope(object):
    def __init__(self):
        """Serial Setup"""
        self.ser = None
        self.s_t = Serial_Tools(self.ser)
        self.s_t.show_messages = False

        self.state = self.s_find_device
        
        self.trace_size = 1024
        self.extra_trace = 16
        self.whole_trace = self.trace_size + self.extra_trace
        self.dump_size = self.trace_size
        self.extra_dump = self.extra_trace
        self.whole_dump = self.trace_size + self.extra_trace

        """Data exposed through API"""
        self.data = {
            "device":{"connected":False, "model":None},
            "accepted_models":["BS0005", "BS0010"],
            "ch":{"active":{}},
            "range":3.5,
            "trigger_level":32000,
            "range_data":{"top":65000, "bot":0}
        }

        self.data['ranges'] = {}
        self.data['ranges']['BS0005'] = {
            1.1 :           "[64]@[56]sn[7e]sn[77]sn[82]s",
            3.5 :           "[64]@[c3]sn[6a]sn[a3]sn[95]s",
            5.2 :           "[64]@[68]sn[44]sn[ff]sn[8a]s",
            11 :            "[64]@[6a]sn[12]sn[8c]sn[ba]s",
            "better_11" :   "[64]@[6f]sn[14]sn[8c]sn[ba]s",
            "max" :         "[64]@[00]sn[00]sn[ff]sn[ff]s"
        }

        self.data['ranges']['BS0010'] = {
            0.520:          "[64]@[40]sn[65]sn[82]sn[6c]s",
            1.1 :           "[64]@[14]sn[61]sn[6f]sn[70]s",
            3.5 :           "[64]@[5c]sn[50]sn[3a]sn[81]s",
            5.2 :           "[64]@[9d]sn[44]sn[38]sn[8d]s",
            11 :            "[64]@[28]sn[1c]sn[c1]sn[b5]s",
            "max" :         "[64]@[00]sn[00]sn[ff]sn[ff]s"
        }
        
        self.data['ch']['active'] = {
            "trace":[],
            "display_trace":[]
        }
        
        self.active = self.data['ch']['active']
        self.ranges = self.data['ranges']
        self.device = self.data['device']

    """ API Functions """
    def step_timebase(self, inc):
        pass
        
    def change_channel(self):
        if self.data['current_channel'] == "a":
            self.data['current_channel'] = "b"
        else:
            self.data['current_channel'] = "a"
            
    def move_range(self, inc):
        td = self.data['range_data']
        td['top'] += inc
        td['bot'] += inc
        if td['top'] > 65500 or td['bot'] < 0:
            td['top'] -= inc
            td['bot'] -= inc
            
        self.state = self.s_change_range
        
    def adjust_range(self, tb, inc):
        td = self.data['range_data']
        td[tb] += inc
        
    def step_trigger_level(self, inc):
        d = self.data
        # Increment the trigger level by 10.
        d['trigger_level'] += inc
        # Check that it's not too high or too low
        if d['trigger_level'] > 65500:
            d['trigger_level'] = 65500
        elif d['trigger_level'] < 0:
            d['trigger_level'] = 0
        
        self.state = self.s_change_trigger

    """ Utility States """
    def s_find_device(self):
        self.ser = self.s_t.find_device()
        if self.ser != None:
            self.data['device']['connected'] = True
            self.state = self.s_check_model
            
        else:
            if self.data['device']['connected']:
                self.data['device']['connected'] = False
            self.state = self.s_find_device

    def s_check_model(self):
        self.ser.read(10000) #Try to get anything in the buffer.
        self.s_t.clear_waiting() #Force the counter to reset.
        self.s_t.issue_wait("?")
        model = (self.ser.read(20)[1:7])
        self.data['device']['model'] = model

        self.dirty = True
        if model in self.data['accepted_models']:
            self.state = self.s_setup_bs
            print self.data['device']['model'] + " Connected."
        else:
            self.state = self.s_check_model
            
    def s_change_trigger(self):
        self.s_t.clear_waiting()
        self.ser.flushInput()
        self.s_t.issue("[68]@[%s]sn[%s]s" % (l_endian_hexify(self.data['trigger_level'])))
        self.s_t.issue_wait(">")
        self.s_t.issue("U")
        self.state = self.s_init_req
        
    def s_change_range(self):
        self.s_t.clear_waiting()
        self.ser.flushInput()
        # Do stuff
        
        self.s_t.issue_wait(">")
        self.s_t.issue("U")
        self.state = self.s_init_req
    
    """ States """
    def s_setup_bs(self):
        siw = self.s_t.issue_wait
        si = self.s_t.issue
        leh = l_endian_hexify
        """ General """
        si("[1c]@[%s]sn[%s]s" % (leh(self.whole_dump))) # d size
        si("[1e]@[00]s") # d mode
        si("[21]@[00]s") # t mode
        si("[08]@[00]sn[00]sn[00]s")# Default spock address
        si("[16]@[01]sn[00]s")# Iterations to 1
        si("[2a]@[%s]sn[%s]s" % (leh(self.whole_trace)))# Post trig cap
        si("[30]@[00]s") # Dump channel?
        si("[31]@[00]s") # Buffer mode
        si("[37]@[01]s") # Analogue chan enable
        si("[26]@[%s]sn[%s]s" % (leh(16))) # Pre trig capture
        si("[2c]@[00]sn[0a]s") # Timeout
        si("[2e]@[90]sn[00]s") # Set clock ticks
        si("[14]@[01]sn[00]s") # Clock scale

        """ Trigger """
        # si("[07]@[00]s") # Trigger on A
        # si("[06]@[00]s") # Set trigger mask to "Don't care about anything"
        # si("[05]@[00]s") # This doesn't matter because we don't care about triggers.
        # si("[28]@[00]sn[00]s") # Pre trigger delay
        # si("[32]@[00]sn[00]sn[00]sn[00]s") # Edge trigger intro, then outro filter.
        # si("[44]@[00]sn[00]s") # Dig comparitor trigger (signed)
        # si("[40]@[00]sn[00]sn[00]sn[00]s") # Stop watch trigger (ticks)
        
        # Example trigger
        si("[06]@[7f]s") # TriggerMask
        si("[05]@[80]s") # TriggerLogic
        si("[32]@[04]s[33]@[00]s") # TriggerIntro
        si("[34]@[04]s[35]@[00]s") # TriggerOutro
        si("[44]@[00]s[45]@[00]s") # TriggerValue
        si("[68]@[f5]s[69]@[18]s") # TriggerLevel
        si("[07]@[21]s") # SpockOption
        siw(">")
        
        """ Range """
        si(self.ranges[self.device['model']][self.data['range']]) #Range entry
        
        """ AWG """
        si("[7c]@[c0]s[86]@[00]s") # AWG on, clock gen off
        siw("U")
        # Synthesize
        si("[46]@[00]sn[00]s[5a]@[00]sn[80]sn[00]sn[00]s") # CV, Op Mode, Phase Ratio
        siw("Y")
        # Translate
        si("[46]@[00]sn[00]s") # CV, Op Mode
        si("[4a]@[e8]sn[03]sn[00]sn[00]sn[00]sn[00]s") # Size, Index, Address
        si("[54]@[ff]sn[ff]sn[00]sn[00]s") # Level, Offset
        si("[5a]@[93]sn[18]sn[04]sn[00]s") # Phase Ratio
        siw("X")
        # Generate
        si("[48]@[f4]sn[80]s[52]@[e8]sn[03]s") # Option, Modulo
        si("[50]@[af]sn[00]s") # Clock ticks
        si("[5e]@[0a]sn[01]sn[01]sn[00]s") # Mark, Space
        si("[78]@[00]sn[7f]s") # Dac Output
        si("[46]@[02]s")
        siw("Z")
        
        """ Update """
        self.s_t.issue_wait(">")
        siw("U")
        self.s_t.clear_waiting()
        self.ser.flushInput()
        
        self.state = self.s_init_req
        
    def s_init_req(self):
        self.s_t.clear_waiting()
        self.ser.flushInput()
        self.s_t.issue_wait(">")
        self.s_t.issue("D")
        
        self.state = self.s_dump
        
    def s_dump(self):
        self.s_t.clear_waiting()
        self.ser.read(33)
        self.s_t.issue_wait(">")
        self.s_t.issue("A")
        
        self.state = self.s_proc_req
        
    def s_proc_req(self):
        self.s_t.clear_waiting()
        self.ser.read(self.extra_dump)
        self.active['trace'] = convert_8bit_dump(self.ser.read(self.dump_size))
        self.s_t.issue_wait(">")
        self.s_t.issue("D")
        
        self.state = self.s_dump
        
    """Data Processing Functions"""

    """Update Functions"""

    def update(self):
        try:
            self.state()
        except serial.SerialException:
            print "Device disconected | Error: SE"
            self.state = self.s_find_device
        except serial.SerialTimeoutException:
            print "Device disconected | Error: STE"
            self.state = self.s_find_device
