from view import View


class Mode(object):
    def __init__(self, name, cfg):
        self.name = name
        self.cfg = cfg

        mode_mch = self.cfg["modes"][self.name]["machine"]
        self.machine = self.cfg["globals"]["machines"][mode_mch]()
        self.callbacks = {}
        self.current_view = None
        self.views = {}

        self.create_callbacks()
        self.create_views()

        nk = self.cfg["modes"][self.name].keys()
        if "current_view" in nk:
            self.switch_view(self.cfg["modes"][self.name]["current_view"])
        else:
            self.switch_view(self.cfg["modes"][self.name]["views"].keys()[0])

        self.mode_request = None

    def handle_action(self, action):
        if action:
            for a in action:
                if a in self.callbacks.keys():
                    for act in self.callbacks[a]:
                        exec(act)
                else:
                    exec(a)

    def switch_view(self, view):
        if view in self.views.keys():
            self.current_view = self.views[view]
            self.current_view.dirty = True
            self.current_view.change_title()
        #find name in the view list, and replace the 'current_view' object with it.

    def to_console(self, output):
        if isinstance(output, list):
            for s in output:
                print str(s) + "\n"
        else:
            print output
            
    def exit(self):
        self.mode_request = "exit"
        
    def handle_key(self, key):
        self.current_view.on_keydown(key)

    def create_callbacks(self):
        if 'callbacks' in self.cfg['modes'][self.name].keys():
            for cb in self.cfg['modes'][self.name]['callbacks'].keys():
                self.callbacks[cb] = []
                for func in self.cfg['modes'][self.name]['callbacks'][cb]:
                    fn = ""
                    bp = func.find("(")
                    func_name = func[:bp]
                    args = func[bp:]
                    if func_name in dir(self.machine):
                        fn = "self.machine." + func_name
                    elif func_name in dir(self):
                        fn = "self." + func_name
                    if fn:
                        ref = compile((fn + args), "string", "exec")
                        self.callbacks[cb].append(ref)
                    else:
                        print "No function called " + func_name + " exists!"
                        print "Check modes -> " + self.name + " -> callbacks"

    def create_views(self):
        for view in self.cfg['modes'][self.name]['views'].keys():
            v = View(view, self)
            self.views[view] = v

    def reset_mode_request(self):
        mr = self.mode_request
        self.mode_request = None
        return mr

    def switch_mode(self, mode):
        self.mode_request = mode

    def update(self):
        self.machine.update()
        self.current_view.data_update()
        return self.reset_mode_request()
