# This should just contain a bunch of colour related helpers.
import math

def RGB_to_HSV(colour):
    r = colour[0]
    g = colour[1]
    b = colour[2]
    
    c_min = min(r,g,b)
    c_max = max(r,g,b)
    # V
    v = float(c_max)
    delta = c_max - c_min
    # S
    if c_max != 0 and delta != 0:
        s = float(delta / float(c_max))
    else:
        s = 0
        h = 0
        return [h,s,v]
    # H
    if r == c_max:
        h = float((g - b) / delta)
    elif g == c_max:
        h = float(2 + (b - r) / float(delta))
    else:
        h = float(4 + (r - g) / float(delta))
    h *= 60.0
    if h < 0:
        h += 360.0
        
    return [h,s,v]
        
def HSV_to_RGB(colour):
    h = colour[0]
    s = colour[1]
    v = colour[2]
    
    # Achromatic
    if s == 0:
        r = g = b = v
        return [r,g,b]
        
    h /= 60.0
    i = math.floor(h)
    f = float(h - i)
    p = float(v * (1 - s))
    q = float(v * (1 - s * f))
    t = float(v * (1 - s * (1 - f)))
    
    if i == 0:
        return [v,t,p]
    elif i == 1:
        return [q,v,p]
    elif i == 2:
        return [p,v,t]
    elif i == 3:
        return [p,q,v]
    elif i == 4:
        return [t,p,v]
    else:
        return [v,p,q]
        
def set_component(col, num, comp):
    hsv = RGB_to_HSV(col)
    hsv[comp] = num
    return HSV_to_RGB(hsv)
    
# Functions for use outside.    

def set_hue(col, hue):
    return set_component(col, hue, 0)
    
def set_saturation(col, sat):
    # Done separately because otherwise you get random colours.
    hsv = RGB_to_HSV(col)
    if hsv[1] == 0:
        return HSV_to_RGB(hsv)
    else:
        hsv[1] == sat
        return HSV_to_RGB(hsv)
    
def set_value(col, val):
    return set_component(col, val, 2)
    
def HSV_alter(op, rgb, comp, val):
    # Convert RGB to HSV
    hsv = RGB_to_HSV(rgb)
    # Check component
    if isinstance(comp, basestring):
        trn = {"h":0, "s":1, "v":2}
        if comp in trn.keys():
            comp = trn[comp]
            
    # Operate
    if op == "add":
        hsv[comp] += val
    elif op == "sub" or op == "subtract":
        hsv[comp] -= val
    elif op == "mult" or op == "multiply":
        hsv[comp] *= val
    elif op == "div" or op == "divide":
        hsv[comp] /= val
        
    return HSV_to_RGB(hsv)
